@@ -1,6 +1,7 @@
 /* typecast.c - basic utility functions related to typecasting
  *
- * Copyright (C) 2003-2010 Federico Di Gregorio <fog@debian.org>
+ * Copyright (C) 2003-2019 Federico Di Gregorio <fog@debian.org>
+ * Copyright (C) 2020-2021 The Psycopg Team
  *
  * This file is part of psycopg.
  *
@@ -31,15 +32,6 @@
 
 /* useful function used by some typecasters */
 
-#ifdef HAVE_MXDATETIME
-static const char *
-skip_until_space(const char *s)
-{
-    while (*s && *s != ' ') s++;
-    return s;
-}
-#endif
-
 static const char *
 skip_until_space2(const char *s, Py_ssize_t *len)
 {
@@ -81,11 +73,11 @@ typecast_parse_date(const char* s, const char** t, Py_ssize_t* len,
         cz += 1;
     }
 
-    /* Is this a BC date?  If so, adjust the year value.  Note that
-     * mx.DateTime numbers BC dates from zero rather than one.  The
-     * Python datetime module does not support BC dates at all. */
+    /* Is this a BC date?  If so, adjust the year value.  However
+     * Python datetime module does not support BC dates, so this will raise
+     * an exception downstream. */
     if (*len >= 2 && s[*len-2] == 'B' && s[*len-1] == 'C')
-        *year = 1 - (*year);
+        *year = -(*year);
 
     if (t != NULL) *t = s;
 
@@ -164,6 +156,9 @@ typecast_parse_time(const char* s, const char** t, Py_ssize_t* len,
         while (usd++ < 6) *us *= 10;
     }
 
+    /* 24:00:00 -> 00:00:00 (ticket #278) */
+    if (*hh == 24) { *hh = 0; }
+
     return cz;
 }
 
@@ -171,11 +166,6 @@ typecast_parse_time(const char* s, const char** t, Py_ssize_t* len,
 #include "psycopg/typecast_basic.c"
 #include "psycopg/typecast_binary.c"
 #include "psycopg/typecast_datetime.c"
-
-#ifdef HAVE_MXDATETIME
-#include "psycopg/typecast_mxdatetime.c"
-#endif
-
 #include "psycopg/typecast_array.c"
 
 static long int typecast_default_DEFAULT[] = {0};
@@ -188,20 +178,13 @@ typecast_UNKNOWN_cast(const char *str, Py_ssize_t len, PyObject *curs)
     Dprintf("typecast_UNKNOWN_cast: str = '%s',"
             " len = " FORMAT_CODE_PY_SSIZE_T, str, len);
 
-    // PostgreSQL returns {} for empty array without explicit type. We convert
-    // that to list in order to handle empty lists.
-    if (len == 2 && str[0] == '{' && str[1] == '}') {
-        return PyList_New(0);
-    }
-
-    Dprintf("typecast_UNKNOWN_cast: fallback to default cast");
-
     return typecast_default.cast(str, len, curs);
 }
 
 #include "psycopg/typecast_builtins.c"
 
 #define typecast_PYDATETIMEARRAY_cast typecast_GENERIC_ARRAY_cast
+#define typecast_PYDATETIMETZARRAY_cast typecast_GENERIC_ARRAY_cast
 #define typecast_PYDATEARRAY_cast typecast_GENERIC_ARRAY_cast
 #define typecast_PYTIMEARRAY_cast typecast_GENERIC_ARRAY_cast
 #define typecast_PYINTERVALARRAY_cast typecast_GENERIC_ARRAY_cast
@@ -209,36 +192,18 @@ typecast_UNKNOWN_cast(const char *str, Py_ssize_t len, PyObject *curs)
 /* a list of initializers, used to make the typecasters accessible anyway */
 static typecastObject_initlist typecast_pydatetime[] = {
     {"PYDATETIME", typecast_DATETIME_types, typecast_PYDATETIME_cast},
+    {"PYDATETIMETZ", typecast_DATETIMETZ_types, typecast_PYDATETIMETZ_cast},
     {"PYTIME", typecast_TIME_types, typecast_PYTIME_cast},
     {"PYDATE", typecast_DATE_types, typecast_PYDATE_cast},
     {"PYINTERVAL", typecast_INTERVAL_types, typecast_PYINTERVAL_cast},
     {"PYDATETIMEARRAY", typecast_DATETIMEARRAY_types, typecast_PYDATETIMEARRAY_cast, "PYDATETIME"},
+    {"PYDATETIMETZARRAY", typecast_DATETIMETZARRAY_types, typecast_PYDATETIMETZARRAY_cast, "PYDATETIMETZ"},
     {"PYTIMEARRAY", typecast_TIMEARRAY_types, typecast_PYTIMEARRAY_cast, "PYTIME"},
     {"PYDATEARRAY", typecast_DATEARRAY_types, typecast_PYDATEARRAY_cast, "PYDATE"},
     {"PYINTERVALARRAY", typecast_INTERVALARRAY_types, typecast_PYINTERVALARRAY_cast, "PYINTERVAL"},
     {NULL, NULL, NULL}
 };
 
-#ifdef HAVE_MXDATETIME
-#define typecast_MXDATETIMEARRAY_cast typecast_GENERIC_ARRAY_cast
-#define typecast_MXDATEARRAY_cast typecast_GENERIC_ARRAY_cast
-#define typecast_MXTIMEARRAY_cast typecast_GENERIC_ARRAY_cast
-#define typecast_MXINTERVALARRAY_cast typecast_GENERIC_ARRAY_cast
-
-/* a list of initializers, used to make the typecasters accessible anyway */
-static typecastObject_initlist typecast_mxdatetime[] = {
-    {"MXDATETIME", typecast_DATETIME_types, typecast_MXDATE_cast},
-    {"MXTIME", typecast_TIME_types, typecast_MXTIME_cast},
-    {"MXDATE", typecast_DATE_types, typecast_MXDATE_cast},
-    {"MXINTERVAL", typecast_INTERVAL_types, typecast_MXINTERVAL_cast},
-    {"MXDATETIMEARRAY", typecast_DATETIMEARRAY_types, typecast_MXDATETIMEARRAY_cast, "MXDATETIME"},
-    {"MXTIMEARRAY", typecast_TIMEARRAY_types, typecast_MXTIMEARRAY_cast, "MXTIME"},
-    {"MXDATEARRAY", typecast_DATEARRAY_types, typecast_MXDATEARRAY_cast, "MXDATE"},
-    {"MXINTERVALARRAY", typecast_INTERVALARRAY_types, typecast_MXINTERVALARRAY_cast, "MXINTERVAL"},
-    {NULL, NULL, NULL}
-};
-#endif
-
 
 /** the type dictionary and associated functions **/
 
@@ -251,11 +216,14 @@ PyObject *psyco_default_binary_cast;
 /* typecast_init - initialize the dictionary and create default types */
 
 RAISES_NEG int
-typecast_init(PyObject *dict)
+typecast_init(PyObject *module)
 {
     int i;
     int rv = -1;
     typecastObject *t = NULL;
+    PyObject *dict = NULL;
+
+    if (!(dict = PyModule_GetDict(module))) { goto exit; }
 
     /* create type dictionary and put it in module namespace */
     if (!(psyco_types = PyDict_New())) { goto exit; }
@@ -267,8 +235,6 @@ typecast_init(PyObject *dict)
     /* insert the cast types into the 'types' dictionary and register them in
        the module dictionary */
     for (i = 0; typecast_builtins[i].name != NULL; i++) {
-        Dprintf("typecast_init: initializing %s", typecast_builtins[i].name);
-
         t = (typecastObject *)typecast_from_c(&(typecast_builtins[i]), dict);
         if (t == NULL) { goto exit; }
         if (typecast_add((PyObject *)t, NULL, 0) < 0) { goto exit; }
@@ -277,32 +243,19 @@ typecast_init(PyObject *dict)
 
         /* export binary object */
         if (typecast_builtins[i].values == typecast_BINARY_types) {
+            Py_INCREF((PyObject *)t);
             psyco_default_binary_cast = (PyObject *)t;
         }
         Py_DECREF((PyObject *)t);
         t = NULL;
     }
 
-    /* create and save a default cast object (but does not register it) */
+    /* create and save a default cast object (but do not register it) */
     psyco_default_cast = typecast_from_c(&typecast_default, dict);
 
     /* register the date/time typecasters with their original names */
-#ifdef HAVE_MXDATETIME
-    if (0 == psyco_typecast_mxdatetime_init()) {
-        for (i = 0; typecast_mxdatetime[i].name != NULL; i++) {
-            Dprintf("typecast_init: initializing %s", typecast_mxdatetime[i].name);
-            t = (typecastObject *)typecast_from_c(&(typecast_mxdatetime[i]), dict);
-            if (t == NULL) { goto exit; }
-            PyDict_SetItem(dict, t->name, (PyObject *)t);
-            Py_DECREF((PyObject *)t);
-            t = NULL;
-        }
-    }
-#endif
-
-    if (0 > psyco_typecast_datetime_init()) { goto exit; }
+    if (0 > typecast_datetime_init()) { goto exit; }
     for (i = 0; typecast_pydatetime[i].name != NULL; i++) {
-        Dprintf("typecast_init: initializing %s", typecast_pydatetime[i].name);
         t = (typecastObject *)typecast_from_c(&(typecast_pydatetime[i]), dict);
         if (t == NULL) { goto exit; }
         PyDict_SetItem(dict, t->name, (PyObject *)t);
@@ -326,23 +279,15 @@ typecast_add(PyObject *obj, PyObject *dict, int binary)
 
     typecastObject *type = (typecastObject *)obj;
 
-    Dprintf("typecast_add: object at %p, values refcnt = "
-        FORMAT_CODE_PY_SSIZE_T,
-        obj, Py_REFCNT(type->values)
-      );
-
     if (dict == NULL)
         dict = (binary ? psyco_binary_types : psyco_types);
 
     len = PyTuple_Size(type->values);
     for (i = 0; i < len; i++) {
         val = PyTuple_GetItem(type->values, i);
-        Dprintf("typecast_add:     adding val: %ld", PyInt_AsLong(val));
         PyDict_SetItem(dict, val, obj);
     }
 
-    Dprintf("typecast_add:     base caster: %p", type->bcast);
-
     return 0;
 }
 
@@ -394,18 +339,11 @@ typecast_cmp(PyObject *obj1, PyObject* obj2)
 static PyObject*
 typecast_richcompare(PyObject *obj1, PyObject* obj2, int opid)
 {
-    PyObject *result = NULL;
     int res = typecast_cmp(obj1, obj2);
 
     if (PyErr_Occurred()) return NULL;
 
-    if ((opid == Py_EQ && res == 0) || (opid != Py_EQ && res != 0))
-        result = Py_True;
-    else
-        result = Py_False;
-
-    Py_INCREF(result);
-    return result;
+    return PyBool_FromLong((opid == Py_EQ && res == 0) || (opid != Py_EQ && res != 0));
 }
 
 static struct PyMemberDef typecastObject_members[] = {
@@ -414,26 +352,27 @@ static struct PyMemberDef typecastObject_members[] = {
     {NULL}
 };
 
-static void
-typecast_dealloc(PyObject *obj)
+static int
+typecast_clear(typecastObject *self)
 {
-    typecastObject *self = (typecastObject*)obj;
-
-    PyObject_GC_UnTrack(self);
-
     Py_CLEAR(self->values);
     Py_CLEAR(self->name);
     Py_CLEAR(self->pcast);
     Py_CLEAR(self->bcast);
+    return 0;
+}
 
-    Py_TYPE(obj)->tp_free(obj);
+static void
+typecast_dealloc(typecastObject *self)
+{
+    PyObject_GC_UnTrack(self);
+    typecast_clear(self);
+    Py_TYPE(self)->tp_free((PyObject *)self);
 }
 
 static int
-typecast_traverse(PyObject *obj, visitproc visit, void *arg)
+typecast_traverse(typecastObject *self, visitproc visit, void *arg)
 {
-    typecastObject *self = (typecastObject*)obj;
-
     Py_VISIT(self->values);
     Py_VISIT(self->name);
     Py_VISIT(self->pcast);
@@ -441,12 +380,6 @@ typecast_traverse(PyObject *obj, visitproc visit, void *arg)
     return 0;
 }
 
-static void
-typecast_del(void *self)
-{
-    PyObject_GC_Del(self);
-}
-
 static PyObject *
 typecast_repr(PyObject *self)
 {
@@ -454,7 +387,7 @@ typecast_repr(PyObject *self)
     PyObject *rv;
 
     Py_INCREF(name);
-    if (!(name = psycopg_ensure_bytes(name))) {
+    if (!(name = psyco_ensure_bytes(name))) {
         return NULL;
     }
 
@@ -479,8 +412,7 @@ typecast_call(PyObject *obj, PyObject *args, PyObject *kwargs)
     // If the string is not a string but a None value we're being called
     // from a Python-defined caster.
     if (!string) {
-        Py_INCREF(Py_None);
-        return Py_None;
+        Py_RETURN_NONE;
     }
 
     return typecast_cast(obj, string, length, cursor);
@@ -489,61 +421,42 @@ typecast_call(PyObject *obj, PyObject *args, PyObject *kwargs)
 PyTypeObject typecastType = {
     PyVarObject_HEAD_INIT(NULL, 0)
     "psycopg2._psycopg.type",
-    sizeof(typecastObject),
-    0,
-
-    typecast_dealloc, /*tp_dealloc*/
+    sizeof(typecastObject), 0,
+    (destructor)typecast_dealloc, /*tp_dealloc*/
     0,          /*tp_print*/
     0,          /*tp_getattr*/
     0,          /*tp_setattr*/
-    typecast_cmp, /*tp_compare*/
+    0,          /*tp_reserved*/
     typecast_repr, /*tp_repr*/
     0,          /*tp_as_number*/
     0,          /*tp_as_sequence*/
     0,          /*tp_as_mapping*/
     0,          /*tp_hash */
-
     typecast_call, /*tp_call*/
     0,          /*tp_str*/
     0,          /*tp_getattro*/
     0,          /*tp_setattro*/
     0,          /*tp_as_buffer*/
-
     Py_TPFLAGS_DEFAULT | Py_TPFLAGS_HAVE_RICHCOMPARE |
       Py_TPFLAGS_HAVE_GC, /*tp_flags*/
     "psycopg type-casting object", /*tp_doc*/
-
-    typecast_traverse, /*tp_traverse*/
-    0,          /*tp_clear*/
-
+    (traverseproc)typecast_traverse, /*tp_traverse*/
+    (inquiry)typecast_clear, /*tp_clear*/
     typecast_richcompare, /*tp_richcompare*/
     0,          /*tp_weaklistoffset*/
-
     0,          /*tp_iter*/
     0,          /*tp_iternext*/
-
-    /* Attribute descriptor and subclassing stuff */
-
-    0, /*tp_methods*/
+    0,          /*tp_methods*/
     typecastObject_members, /*tp_members*/
     0,          /*tp_getset*/
     0,          /*tp_base*/
     0,          /*tp_dict*/
-
     0,          /*tp_descr_get*/
     0,          /*tp_descr_set*/
     0,          /*tp_dictoffset*/
-
-    0, /*tp_init*/
-    0, /*tp_alloc  will be set to PyType_GenericAlloc in module init*/
-    0, /*tp_new*/
-    typecast_del, /*tp_free  Low-level free-memory routine */
-    0,          /*tp_is_gc For PyObject_IS_GC */
-    0,          /*tp_bases*/
-    0,          /*tp_mro method resolution order */
-    0,          /*tp_cache*/
-    0,          /*tp_subclasses*/
-    0           /*tp_weaklist*/
+    0,          /*tp_init*/
+    0,          /*tp_alloc*/
+    0,          /*tp_new*/
 };
 
 static PyObject *
@@ -554,9 +467,6 @@ typecast_new(PyObject *name, PyObject *values, PyObject *cast, PyObject *base)
     obj = PyObject_GC_New(typecastObject, &typecastType);
     if (obj == NULL) return NULL;
 
-    Dprintf("typecast_new: new type at = %p, refcnt = " FORMAT_CODE_PY_SSIZE_T,
-      obj, Py_REFCNT(obj));
-
     Py_INCREF(values);
     obj->values = values;
 
@@ -583,8 +493,6 @@ typecast_new(PyObject *name, PyObject *values, PyObject *cast, PyObject *base)
 
     PyObject_GC_Track(obj);
 
-    Dprintf("typecast_new: typecast object created at %p", obj);
-
     return (PyObject *)obj;
 }
 
@@ -690,12 +598,7 @@ typecast_cast(PyObject *obj, const char *str, Py_ssize_t len, PyObject *curs)
          * Notice that this way it is about impossible to create a python
          * typecaster on a binary type. */
         if (str) {
-#if PY_MAJOR_VERSION < 3
-            s = PyString_FromStringAndSize(str, len);
-#else
-            s = PyUnicode_Decode(str, len,
-                ((cursorObject *)curs)->conn->codec, NULL);
-#endif
+            s = conn_decode(((cursorObject *)curs)->conn, str, len);
         }
         else {
             Py_INCREF(Py_None);
