@@ -3,7 +3,7 @@
 # test_cursor.py - unit test for cursor attributes
 #
 # Copyright (C) 2010-2019 Daniele Varrazzo  <daniele.varrazzo@gmail.com>
-# Copyright (C) 2020 The Psycopg Team
+# Copyright (C) 2020-2021 The Psycopg Team
 #
 # psycopg2 is free software: you can redistribute it and/or modify it
 # under the terms of the GNU Lesser General Public License as published
@@ -39,7 +39,6 @@ from .testutils import (ConnectingTestCase, skip_before_postgres,
     skip_if_windows, skip_if_crdb, crdb_version)
 
 import psycopg2.extras
-from psycopg2.compat import text_type
 
 
 class CursorTests(ConnectingTestCase):
@@ -75,36 +74,36 @@ class CursorTests(ConnectingTestCase):
         # test consistency between execute and mogrify.
 
         # unicode query containing only ascii data
-        cur.execute(u"SELECT 'foo';")
+        cur.execute("SELECT 'foo';")
         self.assertEqual('foo', cur.fetchone()[0])
-        self.assertEqual(b"SELECT 'foo';", cur.mogrify(u"SELECT 'foo';"))
+        self.assertEqual(b"SELECT 'foo';", cur.mogrify("SELECT 'foo';"))
 
         conn.set_client_encoding('UTF8')
-        snowman = u"\u2603"
+        snowman = "\u2603"
 
         def b(s):
-            if isinstance(s, text_type):
+            if isinstance(s, str):
                 return s.encode('utf8')
             else:
                 return s
 
         # unicode query with non-ascii data
-        cur.execute(u"SELECT '%s';" % snowman)
+        cur.execute(f"SELECT '{snowman}';")
         self.assertEqual(snowman.encode('utf8'), b(cur.fetchone()[0]))
-        self.assertQuotedEqual(("SELECT '%s';" % snowman).encode('utf8'),
-            cur.mogrify(u"SELECT '%s';" % snowman))
+        self.assertQuotedEqual(f"SELECT '{snowman}';".encode('utf8'),
+            cur.mogrify(f"SELECT '{snowman}';"))
 
         # unicode args
         cur.execute("SELECT %s;", (snowman,))
         self.assertEqual(snowman.encode("utf-8"), b(cur.fetchone()[0]))
-        self.assertQuotedEqual(("SELECT '%s';" % snowman).encode('utf8'),
+        self.assertQuotedEqual(f"SELECT '{snowman}';".encode('utf8'),
             cur.mogrify("SELECT %s;", (snowman,)))
 
         # unicode query and args
-        cur.execute(u"SELECT %s;", (snowman,))
+        cur.execute("SELECT %s;", (snowman,))
         self.assertEqual(snowman.encode("utf-8"), b(cur.fetchone()[0]))
-        self.assertQuotedEqual(("SELECT '%s';" % snowman).encode('utf8'),
-            cur.mogrify(u"SELECT %s;", (snowman,)))
+        self.assertQuotedEqual(f"SELECT '{snowman}';".encode('utf8'),
+            cur.mogrify("SELECT %s;", (snowman,)))
 
     def test_mogrify_decimal_explodes(self):
         conn = self.conn
@@ -250,6 +249,16 @@ class CursorTests(ConnectingTestCase):
 
         self.assertEqual(description, unpickled)
 
+    def test_column_refcount(self):
+        # Reproduce crash describe in ticket #1252
+        from psycopg2.extensions import Column
+
+        def do_stuff():
+            _ = Column(name='my_column')
+
+        for _ in range(1000):
+            do_stuff()
+
     def test_bad_subclass(self):
         # check that we get an error message instead of a segfault
         # for badly written subclasses.
@@ -283,12 +292,12 @@ class CursorTests(ConnectingTestCase):
         cur = self.conn.cursor()
 
         # Set up the temporary function
-        cur.execute('''
-            CREATE FUNCTION %s(%s INT)
+        cur.execute(f'''
+            CREATE FUNCTION {procname}({escaped_paramname} INT)
             RETURNS INT AS
                 'SELECT $1 * $1'
             LANGUAGE SQL
-        ''' % (procname, escaped_paramname))
+        ''')
 
         # Make sure callproc works right
         cur.callproc(procname, {paramname: 2})
@@ -299,7 +308,7 @@ class CursorTests(ConnectingTestCase):
             ({paramname: 2, 'foo': 'bar'}, psycopg2.ProgrammingError),
             ({paramname: '2'}, psycopg2.ProgrammingError),
             ({paramname: 'two'}, psycopg2.ProgrammingError),
-            ({u'bj\xc3rn': 2}, psycopg2.ProgrammingError),
+            ({'bj\xc3rn': 2}, psycopg2.ProgrammingError),
             ({3: 2}, TypeError),
             ({self: 2}, TypeError),
         ]
@@ -574,8 +583,7 @@ class NamedCursorTests(ConnectingTestCase):
         time.sleep(0.2)
         t2 = next(i)[0]
         self.assert_((t2 - t1).microseconds * 1e-6 < 0.1,
-            "named cursor records fetched in 2 roundtrips (delta: %s)"
-            % (t2 - t1))
+            f"named cursor records fetched in 2 roundtrips (delta: {t2 - t1})")
 
     @skip_before_postgres(8, 0)
     def test_iter_named_cursor_default_itersize(self):
