@@ -3,7 +3,7 @@
 # test_ctxman.py - unit test for connection and cursor used as context manager
 #
 # Copyright (C) 2012-2019 Daniele Varrazzo  <daniele.varrazzo@gmail.com>
-# Copyright (C) 2020 The Psycopg Team
+# Copyright (C) 2020-2021 The Psycopg Team
 #
 # psycopg2 is free software: you can redistribute it and/or modify it
 # under the terms of the GNU Lesser General Public License as published
@@ -27,7 +27,7 @@ import psycopg2
 import psycopg2.extensions as ext
 
 import unittest
-from .testutils import ConnectingTestCase, skip_before_postgres
+from .testutils import ConnectingTestCase, skip_before_postgres, skip_if_crdb
 
 
 class WithTestCase(ConnectingTestCase):
@@ -117,7 +117,7 @@ class WithConnectionTestCase(WithTestCase):
         class MyConn(ext.connection):
             def commit(self):
                 commits.append(None)
-                super(MyConn, self).commit()
+                super().commit()
 
         with self.connect(connection_factory=MyConn) as conn:
             curs = conn.cursor()
@@ -136,7 +136,7 @@ class WithConnectionTestCase(WithTestCase):
         class MyConn(ext.connection):
             def rollback(self):
                 rollbacks.append(None)
-                super(MyConn, self).rollback()
+                super().rollback()
 
         try:
             with self.connect(connection_factory=MyConn) as conn:
@@ -155,6 +155,93 @@ class WithConnectionTestCase(WithTestCase):
         curs.execute("select * from test_with")
         self.assertEqual(curs.fetchall(), [])
 
+    def test_cant_reenter(self):
+        raised_ok = False
+        with self.conn:
+            try:
+                with self.conn:
+                    pass
+            except psycopg2.ProgrammingError:
+                raised_ok = True
+
+        self.assert_(raised_ok)
+
+        # Still good
+        with self.conn:
+            pass
+
+    def test_with_autocommit(self):
+        self.conn.autocommit = True
+        self.assertEqual(
+            self.conn.info.transaction_status, ext.TRANSACTION_STATUS_IDLE
+        )
+        with self.conn:
+            curs = self.conn.cursor()
+            curs.execute("insert into test_with values (1)")
+            self.assertEqual(
+                self.conn.info.transaction_status,
+                ext.TRANSACTION_STATUS_INTRANS,
+            )
+
+        self.assertEqual(
+            self.conn.info.transaction_status, ext.TRANSACTION_STATUS_IDLE
+        )
+        curs.execute("select count(*) from test_with")
+        self.assertEqual(curs.fetchone()[0], 1)
+        self.assertEqual(
+            self.conn.info.transaction_status, ext.TRANSACTION_STATUS_IDLE
+        )
+
+    def test_with_autocommit_pyerror(self):
+        self.conn.autocommit = True
+        raised_ok = False
+        try:
+            with self.conn:
+                curs = self.conn.cursor()
+                curs.execute("insert into test_with values (2)")
+                self.assertEqual(
+                    self.conn.info.transaction_status,
+                    ext.TRANSACTION_STATUS_INTRANS,
+                )
+                1 / 0
+        except ZeroDivisionError:
+            raised_ok = True
+
+        self.assert_(raised_ok)
+        self.assertEqual(
+            self.conn.info.transaction_status, ext.TRANSACTION_STATUS_IDLE
+        )
+        curs.execute("select count(*) from test_with")
+        self.assertEqual(curs.fetchone()[0], 0)
+        self.assertEqual(
+            self.conn.info.transaction_status, ext.TRANSACTION_STATUS_IDLE
+        )
+
+    def test_with_autocommit_pgerror(self):
+        self.conn.autocommit = True
+        raised_ok = False
+        try:
+            with self.conn:
+                curs = self.conn.cursor()
+                curs.execute("insert into test_with values (2)")
+                self.assertEqual(
+                    self.conn.info.transaction_status,
+                    ext.TRANSACTION_STATUS_INTRANS,
+                )
+                curs.execute("insert into test_with values ('x')")
+        except psycopg2.errors.InvalidTextRepresentation:
+            raised_ok = True
+
+        self.assert_(raised_ok)
+        self.assertEqual(
+            self.conn.info.transaction_status, ext.TRANSACTION_STATUS_IDLE
+        )
+        curs.execute("select count(*) from test_with")
+        self.assertEqual(curs.fetchone()[0], 0)
+        self.assertEqual(
+            self.conn.info.transaction_status, ext.TRANSACTION_STATUS_IDLE
+        )
+
 
 class WithCursorTestCase(WithTestCase):
     def test_with_ok(self):
@@ -195,7 +282,7 @@ class WithCursorTestCase(WithTestCase):
         class MyCurs(ext.cursor):
             def close(self):
                 closes.append(None)
-                super(MyCurs, self).close()
+                super().close()
 
         with self.conn.cursor(cursor_factory=MyCurs) as curs:
             self.assert_(isinstance(curs, MyCurs))
@@ -203,6 +290,7 @@ class WithCursorTestCase(WithTestCase):
         self.assert_(curs.closed)
         self.assert_(closes)
 
+    @skip_if_crdb("named cursor")
     def test_exception_swallow(self):
         # bug #262: __exit__ calls cur.close() that hides the exception
         # with another error.
@@ -216,6 +304,7 @@ class WithCursorTestCase(WithTestCase):
         else:
             self.fail("where is my exception?")
 
+    @skip_if_crdb("named cursor")
     @skip_before_postgres(8, 2)
     def test_named_with_noop(self):
         with self.conn.cursor('named'):
